import requests

DEFAULT_QUERY = """
{{
  user(login: "{username}") {{
    starredRepositories(first: 100, after: "{after}", orderBy: {{direction: DESC, field: STARRED_AT}}) {{
      totalCount
      nodes {{
        name
        nameWithOwner
        description
        url
        stargazerCount
        forkCount
        pushedAt
        updatedAt
        languages(first: 1, orderBy: {{field: SIZE, direction: DESC}}) {{
          edges {{
            node {{
              id
              name
            }}
          }}
        }}
      }}
      pageInfo {{
        endCursor
        hasNextPage
      }}
    }}
  }}
}}
"""


class Repository:
    def __init__(self, name, description, language, url, stargazer_count):
        self.name = name
        self.description = description
        self.language = language
        self.url = url
        self.stargazer_count = stargazer_count


def get_user_starred_by_username(token, username, query=None):
    items = []
    headers = {"Authorization": f"Bearer {token}"}
    if query is None:
        query = DEFAULT_QUERY.format(username=username, after='')
    r = requests.post('https://api.github.com/graphql', json={'query': query}, headers=headers)
    if r.status_code != 200:
        raise Exception(f'Query failed to run by returning code: {r.status_code}, query: {query}, result: {r.json()}')

    data = r.json()
    has_next = data['data']['user']['starredRepositories']['pageInfo']['hasNextPage']
    end_cursor = data['data']['user']['starredRepositories']['pageInfo']['endCursor']
    # total_count = data['data']['user']['starredRepositories']['totalCount']

    for repo in data['data']['user']['starredRepositories']['nodes']:
        name = repo['nameWithOwner']
        description = repo['description'][:30] if repo['description'] else ''
        language = repo['languages']['edges'][0]['node']['name'] if repo['languages']['edges'] else ''
        url = repo['url']
        stargazer_count = repo['stargazerCount']
        items.append(Repository(name, description, language, url, stargazer_count))

    if has_next:
        query = DEFAULT_QUERY.format(username=username, after=end_cursor)
        items.extend(get_user_starred_by_username(token, username, query))
    return items



from setuptools import setup, find_packages
from os import path
from starred import VERSION


this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()


setup(
    name='starred',
    version=VERSION,
    url='https://github.com/maguowei/starred',
    license='MIT',
    author='maguowei',
    author_email='imaguowei@gmail.com',
    keywords='GitHub starred',
    description='creating your own Awesome List used GitHub stars!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    platforms='any',
    python_requires='>=3.7',
    install_requires=[
        'click==8.1.3',
        'requests==2.27.1',
        'github3.py==3.2.0',
    ],
    entry_points={
        'console_scripts': [
            'starred=starred.starred:starred'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from io import BytesIO
from collections import OrderedDict
import click
from github3 import GitHub
from github3.exceptions import NotFoundError
from . import VERSION
from .github import get_user_starred_by_username


desc = '''# Awesome Stars [![Awesome](https://cdn.rawgit.com/sindresorhus/awesome/d730\
5f38d29fed78fa85652e3a63e154dd8e8829/media/badge.svg)](https://github.com/sindresorhus/awesome)

> A curated list of my GitHub stars!  Generated by [starred](https://github.com/maguowei/starred)


## Contents
'''

license_ = '''
## License

[![CC0](http://mirrors.creativecommons.org/presskit/buttons/88x31/svg/cc-zero.svg)]\
(https://creativecommons.org/publicdomain/zero/1.0/)

To the extent possible under law, [{username}](https://github.com/{username})\
 has waived all copyright and related or neighboring rights to this work.
'''

html_escape_table = {
    ">": "&gt;",
    "<": "&lt;",
}


def html_escape(text):
    """Produce entities within text."""
    return "".join(html_escape_table.get(c, c) for c in text)


@click.command()
@click.option('--username', envvar='USER', required=True, help='GitHub username')
@click.option('--token', envvar='GITHUB_TOKEN', required=True, help='GitHub token')
@click.option('--sort',  is_flag=True, help='sort by language')
@click.option('--repository', default='', help='repository name')
@click.option('--message', default='update stars', help='commit message')
@click.version_option(version=VERSION, prog_name='starred')
def starred(username, token, sort, repository, message):
    """GitHub starred

    creating your own Awesome List used GitHub stars!

    example:
        starred --username maguowei --token=xxxxxxxx --sort > README.md
    """
    if repository:
        file = BytesIO()
        sys.stdout = file
    else:
        file = None

    stars = get_user_starred_by_username(token, username)
    click.echo(desc)
    repo_dict = {}

    for s in stars:
        language = s.language or 'Others'
        description = html_escape(s.description).replace('\n', '') if s.description else ''
        if language not in repo_dict:
            repo_dict[language] = []
        repo_dict[language].append([s.name, s.url, description.strip()])

    if sort:
        repo_dict = OrderedDict(sorted(repo_dict.items(), key=lambda l: l[0]))

    for language in repo_dict.keys():
        data = u'  - [{}](#{})'.format(language, '-'.join(language.lower().split()))
        click.echo(data)
    click.echo('')

    for language in repo_dict:
        click.echo('## {} \n'.format(language.replace('#', '# #')))
        for repo in repo_dict[language]:
            data = u'- [{}]({}) - {}'.format(*repo)
            click.echo(data)
        click.echo('')

    click.echo(license_.format(username=username))

    if file:
        gh = GitHub(token=token)
        try:
            rep = gh.repository(username, repository)
            readme = rep.readme()
            readme.update(message, file.getvalue())
        except NotFoundError:
            rep = gh.create_repository(repository, 'A curated list of my GitHub stars!')
            rep.create_file('README.md', 'starred initial commit', file.getvalue())
        click.launch(rep.html_url)


if __name__ == '__main__':
    starred()
